@@ -404,7 +404,7 @@ gpg_ctx_get_diagnostics (struct _GpgCtx *gpg)
 		g_byte_array_append (gpg->diagbuf, "", 1);
 	}
 	
-	return gpg->diagbuf->data;
+	return (const char *) gpg->diagbuf->data;
 }
 
 static void
@@ -794,46 +794,71 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 		
 		g_free (gpg->need_id);
 		gpg->need_id = userid;
-	} else if (!strncmp (status, "GET_HIDDEN passphrase.enter", 27)) {
+	} else if (!strncmp (status, "NEED_PASSPHRASE_PIN ", 20)) {
+		char *userid;
+		
+		status += 20;
+		
+		status = next_token (status, &userid);
+		if (!userid) {
+			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
+					     _("Failed to parse gpg passphrase request."));
+			return -1;
+		}
+		
+		g_free (gpg->need_id);
+		gpg->need_id = userid;
+	} else if (!strncmp (status, "GET_HIDDEN ", 11)) {
 		char *prompt, *passwd;
 		const char *name;
-
-		if (gpg->need_id) {
-			name = g_hash_table_lookup (gpg->userid_hint, gpg->need_id);
-			if (!name)
-				name = gpg->need_id;
-
-			prompt = g_strdup_printf (_("You need a passphrase to unlock the key for\n"
-						"user: \"%s\""), name);
-		} else {
+		guint32 flags;
+		
+		status += 11;
+		
+		if (gpg->need_id && !(name = g_hash_table_lookup (gpg->userid_hint, gpg->need_id)))
+			name = gpg->need_id;
+		else
 			name = "";
+		
+		if (!strncmp (status, "passphrase.pin.ask", 18)) {
+			prompt = g_strdup_printf (_("You need a PIN to unlock the key for your\n"
+						    "SmartCard: \"%s\""), name);
+		} else if (!strncmp (status, "passphrase.enter", 16)) {
 			prompt = g_strdup_printf (_("You need a passphrase to unlock the key for\n"
-						"user: \"%s\""), name);
+						    "user: \"%s\""), name);
+		} else {
+			next_token (status, &prompt);
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					      _("Unexpected request from GnuPG for `%s'"), prompt);
+			g_free (prompt);
+			return -1;
 		}
 		
-		if ((passwd = camel_session_get_password (gpg->session, NULL, NULL, prompt,  gpg->need_id, CAMEL_SESSION_PASSWORD_SECRET, ex)) && !gpg->utf8) {
-			char *opasswd = passwd;
-			
-			if ((passwd = g_locale_to_utf8 (passwd, -1, &nread, &nwritten, NULL))) {
-				memset (opasswd, 0, strlen (opasswd));
-				g_free (opasswd);
-			} else {
-				passwd = opasswd;
+		flags = CAMEL_SESSION_PASSWORD_SECRET | CAMEL_SESSION_PASSPHRASE;
+		if ((passwd = camel_session_get_password (gpg->session, NULL, NULL, prompt, gpg->need_id, flags, ex))) {
+			if (!gpg->utf8) {
+				char *opasswd = passwd;
+				
+				if ((passwd = g_locale_to_utf8 (passwd, -1, &nread, &nwritten, NULL))) {
+					memset (opasswd, 0, strlen (opasswd));
+					g_free (opasswd);
+				} else {
+					passwd = opasswd;
+				}
 			}
-		}
-		g_free (prompt);
-		
-		if (passwd == NULL) {
+			
+			gpg->passwd = g_strdup_printf ("%s\n", passwd);
+			memset (passwd, 0, strlen (passwd));
+			g_free (passwd);
+			
+			gpg->send_passwd = TRUE;
+		} else {
 			if (!camel_exception_is_set (ex))
 				camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, _("Cancelled."));
 			return -1;
 		}
 		
-		gpg->passwd = g_strdup_printf ("%s\n", passwd);
-		memset (passwd, 0, strlen (passwd));
-		g_free (passwd);
-		
-		gpg->send_passwd = TRUE;
+		g_free (prompt);
 	} else if (!strncmp (status, "GOOD_PASSPHRASE", 15)) {
 		gpg->bad_passwds = 0;
 	} else if (!strncmp (status, "BAD_PASSPHRASE", 14)) {
@@ -1362,17 +1387,14 @@ swrite (CamelMimePart *sigpart)
 	char *template;
 	int fd, ret;
 	
-	template = g_build_filename (g_get_tmp_dir (),
-				     "evolution-pgp.XXXXXX",
-				     NULL);
-	fd = g_mkstemp (template);
-	if (fd == -1) {
+	template = g_build_filename (g_get_tmp_dir (), "evolution-pgp.XXXXXX", NULL);
+	if ((fd = g_mkstemp (template)) == -1) {
 		g_free (template);
 		return NULL;
 	}
-
+	
 	/* TODO: This should probably just write the decoded message content out, not the part + headers */
-
+	
 	ostream = camel_stream_fs_new_with_fd (fd);
 	ret = camel_data_wrapper_write_to_stream((CamelDataWrapper *)sigpart, ostream);
 	if (ret != -1) {
@@ -1380,9 +1402,9 @@ swrite (CamelMimePart *sigpart)
 		if (ret != -1)
 			ret = camel_stream_close (ostream);
 	}
-
+	
 	camel_object_unref(ostream);
-
+	
 	if (ret == -1) {
 		g_unlink (template);
 		g_free (template);
@@ -1442,15 +1464,13 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 		camel_data_wrapper_decode_to_stream (content, istream);
 		camel_stream_reset(istream);
 		sigpart = NULL;
-			
 	} else {
 		/* Invalid Mimetype */
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 			      _("Cannot verify message signature: Incorrect message format"));
 		return NULL;
 	}
-    
-
+	
 	/* Now start the real work of verifying the message */
 #ifdef GPG_LOG
 	if (camel_debug_start("gpg:sign")) {
