@@ -5,7 +5,7 @@
  *  Authors: Michael Zucchi <notzed@ximian.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -23,25 +23,25 @@
 #include <config.h>
 #endif
 
-#include "camel-mbox-summary.h"
-#include "camel/camel-mime-message.h"
-#include "camel/camel-operation.h"
-
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
-#include <sys/uio.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "camel-mbox-summary.h"
-#include "camel/camel-file-utils.h"
-#include "camel/camel-mime-message.h"
-#include "camel/camel-operation.h"
+#include <glib.h>
+#include <glib/gstdio.h>
+
+#include "camel-file-utils.h"
 #include "camel-i18n.h"
+#include "camel-mime-message.h"
+#include "camel-operation.h"
+#include "camel-private.h"
+
+#include "camel-mbox-summary.h"
 
 #define io(x)
 #define d(x) /*(printf("%s(%d): ", __FILE__, __LINE__),(x))*/
@@ -107,7 +107,7 @@ mbox_info_set_user_flag(CamelMessageInfo *mi, const char *name, gboolean value)
 
 	res = ((CamelFolderSummaryClass *)camel_mbox_summary_parent)->info_set_user_flag(mi, name, value);
 	if (res)
-		((CamelLocalMessageInfo *)mi)->info.flags |= CAMEL_MESSAGE_FOLDER_FLAGGED|CAMEL_MESSAGE_FOLDER_XEVCHANGE;
+		((CamelLocalMessageInfo *)mi)->info.flags |= CAMEL_MESSAGE_FOLDER_FLAGGED;
 
 	return res;
 }
@@ -119,7 +119,7 @@ mbox_info_set_user_tag(CamelMessageInfo *mi, const char *name, const char *value
 
 	res = ((CamelFolderSummaryClass *)camel_mbox_summary_parent)->info_set_user_tag(mi, name, value);
 	if (res)
-		((CamelLocalMessageInfo *)mi)->info.flags |= CAMEL_MESSAGE_FOLDER_FLAGGED|CAMEL_MESSAGE_FOLDER_XEVCHANGE;
+		((CamelLocalMessageInfo *)mi)->info.flags |= CAMEL_MESSAGE_FOLDER_FLAGGED;
 
 	return res;
 }
@@ -415,7 +415,7 @@ summary_update(CamelLocalSummary *cls, off_t offset, CamelFolderChangeInfo *chan
 
 	camel_operation_start(NULL, _("Storing folder"));
 
-	fd = open(cls->folder_path, O_RDONLY);
+	fd = g_open(cls->folder_path, O_LARGEFILE | O_RDONLY | O_BINARY, 0);
 	if (fd == -1) {
 		d(printf("%s failed to open: %s\n", cls->folder_path, strerror (errno)));
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -588,7 +588,7 @@ mbox_summary_sync_full(CamelMboxSummary *mbs, gboolean expunge, CamelFolderChang
 
 	camel_operation_start(NULL, _("Storing folder"));
 
-	fd = open(cls->folder_path, O_RDONLY);
+	fd = g_open(cls->folder_path, O_LARGEFILE | O_RDONLY | O_BINARY, 0);
 	if (fd == -1) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Could not open file: %s: %s"),
@@ -600,7 +600,7 @@ mbox_summary_sync_full(CamelMboxSummary *mbs, gboolean expunge, CamelFolderChang
 	tmpname = g_alloca (strlen (cls->folder_path) + 5);
 	sprintf (tmpname, "%s.tmp", cls->folder_path);
 	d(printf("Writing tmp file to %s\n", tmpname));
-	fdout = open(tmpname, O_WRONLY|O_CREAT|O_TRUNC, 0600);
+	fdout = g_open(tmpname, O_LARGEFILE|O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0600);
 	if (fdout == -1) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Cannot open temporary mailbox: %s"),
@@ -633,7 +633,11 @@ mbox_summary_sync_full(CamelMboxSummary *mbs, gboolean expunge, CamelFolderChang
 
 	/* this should probably either use unlink/link/unlink, or recopy over
 	   the original mailbox, for various locking reasons/etc */
-	if (rename(tmpname, cls->folder_path) == -1) {
+#ifdef G_OS_WIN32
+	if (g_file_test(cls->folder_path,G_FILE_TEST_IS_REGULAR) && g_remove(cls->folder_path) == -1)
+		g_warning ("Cannot remove %s: %s", cls->folder_path, g_strerror (errno));
+#endif
+	if (g_rename(tmpname, cls->folder_path) == -1) {
 		g_warning("Cannot rename folder: %s", strerror (errno));
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Could not rename folder: %s"),
@@ -653,7 +657,7 @@ mbox_summary_sync_full(CamelMboxSummary *mbs, gboolean expunge, CamelFolderChang
 		close(fdout);
 	
 	if (tmpname)
-		unlink(tmpname);
+		g_unlink(tmpname);
 
 	camel_operation_end(NULL);
 
@@ -669,7 +673,7 @@ mbox_summary_sync_quick(CamelMboxSummary *mbs, gboolean expunge, CamelFolderChan
 	CamelMimeParser *mp = NULL;
 	int i, count;
 	CamelMboxMessageInfo *info = NULL;
-	int fd = -1;
+	int fd = -1, pfd;
 	char *xevnew, *xevtmp;
 	const char *xev;
 	int len;
@@ -679,7 +683,7 @@ mbox_summary_sync_quick(CamelMboxSummary *mbs, gboolean expunge, CamelFolderChan
 
 	camel_operation_start(NULL, _("Storing folder"));
 
-	fd = open(cls->folder_path, O_RDWR);
+	fd = g_open(cls->folder_path, O_LARGEFILE|O_RDWR|O_BINARY, 0);
 	if (fd == -1) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Could not open file: %s: %s"),
@@ -689,10 +693,20 @@ mbox_summary_sync_quick(CamelMboxSummary *mbs, gboolean expunge, CamelFolderChan
 		return -1;
 	}
 
+	/* need to dup since mime parser closes its fd once it is finalised */
+	pfd = dup(fd);
+	if (pfd == -1) {
+		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
+				     _("Could not store folder: %s"),
+				     g_strerror(errno));
+		close(fd);
+		return -1;
+	}
+
 	mp = camel_mime_parser_new();
 	camel_mime_parser_scan_from(mp, TRUE);
 	camel_mime_parser_scan_pre_from(mp, TRUE);
-	camel_mime_parser_init_with_fd(mp, fd);
+	camel_mime_parser_init_with_fd(mp, pfd);
 
 	count = camel_folder_summary_count(s);
 	for (i = 0; i < count; i++) {
@@ -749,8 +763,6 @@ mbox_summary_sync_quick(CamelMboxSummary *mbs, gboolean expunge, CamelFolderChan
 		xevtmp = camel_header_unfold(xevnew);
 		/* the raw header contains a leading ' ', so (dis)count that too */
 		if (strlen(xev)-1 != strlen(xevtmp)) {
-			printf ("strlen(xev)-1 = %d; strlen(xevtmp) = %d\n", strlen(xev)-1, strlen(xevtmp));
-			printf ("xev='%s'; xevtmp='%s'\n", xev, xevtmp);
 			g_free(xevnew);
 			g_free(xevtmp);
 			g_warning("Hmm, the xev headers shouldn't have changed size, but they did");
@@ -1091,32 +1103,32 @@ static struct {
 static void
 encode_status(guint32 flags, char status[8])
 {
+	size_t i;
 	char *p;
-	int i;
-
+	
 	p = status;
-	for (i=0;i<sizeof(status_flags)/sizeof(status_flags[0]);i++)
+	for (i = 0; i < G_N_ELEMENTS (status_flags); i++)
 		if (status_flags[i].flag & flags)
 			*p++ = status_flags[i].tag;
 	*p++ = 'O';
-	*p=0;
+	*p = '\0';
 }
 
 static guint32
 decode_status(const char *status)
 {
 	const char *p;
-	char c;
 	guint32 flags = 0;
-	int i;
-
+	size_t i;
+	char c;
+	
 	p = status;
 	while ((c = *p++)) {
-		for (i=0;i<sizeof(status_flags)/sizeof(status_flags[0]);i++)
-			if (status_flags[i].tag == *p)
+		for (i = 0; i < G_N_ELEMENTS (status_flags); i++)
+			if (status_flags[i].tag == c)
 				flags |= status_flags[i].flag;
 	}
-
+	
 	return flags;
 }
 
